-- The SQL*Plus script object_user_schema.sql performs the following:
--   1. Creates the object_user.
--   2. Creates the database objects.
--   3. Populates the database tables with example data.

-- This script should be run by the system user (or the DBA).
CONNECT system/manager;

-- drop object_user
DROP USER object_user CASCADE;

-- create object_user
CREATE USER object_user IDENTIFIED BY object_password;

-- allow object_user to connect and create database objects
GRANT connect, resource TO object_user;

-- connect as the object_user
CONNECT object_user/object_password;

-- create the user defined types
CREATE TYPE t_address AS OBJECT (
  street VARCHAR2(15),
  city   VARCHAR2(15),
  state  CHAR(2),
  zip    VARCHAR2(9)
);
/

CREATE OR REPLACE TYPE t_customer AS OBJECT (
  id         NUMBER,
  first_name VARCHAR2(10),
  last_name  VARCHAR2(10),
  dob        DATE,
  phone      VARCHAR2(15),
  address    t_address,

  -- the function get_age() returns the age of the customer in years
  MEMBER FUNCTION get_age RETURN INTEGER
);
/

CREATE OR REPLACE TYPE BODY t_customer AS

  -- the function get_age() returns the age of the customer in years
  MEMBER FUNCTION get_age RETURN INTEGER IS

    age INTEGER;

  BEGIN

    -- calculate the age in years
    SELECT ROUND(((sysdate - dob) / 365), 0) INTO age FROM dual;

    RETURN age;

  END;

END;
/

CREATE TYPE t_address2 AS VARRAY(3) OF VARCHAR2(50);
/

CREATE TYPE t_address3 AS VARRAY(3) OF t_address;
/

CREATE TYPE t_address4 AS TABLE OF VARCHAR2(50);
/

CREATE TYPE t_product AS OBJECT (
  id          NUMBER,
  name        VARCHAR2(50),
  description VARCHAR2(500),
  price       NUMBER(10, 2)
);
/


-- create the tables
CREATE TABLE customers (
  customer t_customer
);

CREATE TABLE customers2 OF t_customer;

CREATE TABLE customers3 (
  id         NUMBER CONSTRAINT customers3_pk PRIMARY KEY,
  first_name VARCHAR2(10),
  last_name  VARCHAR2(10),
  addresses  t_address2
);

CREATE TABLE customers4 (
  id         NUMBER CONSTRAINT customers4_pk PRIMARY KEY,
  first_name VARCHAR2(10),
  last_name  VARCHAR2(10),
  addresses  t_address3
);

CREATE TABLE customers5 (
  id         NUMBER CONSTRAINT customers5_pk PRIMARY KEY,
  first_name VARCHAR2(10),
  last_name  VARCHAR2(10),
  addresses  t_address4
)
NESTED TABLE
  addresses
STORE AS
  nested_addresses;

CREATE TABLE products OF t_product;

CREATE TABLE purchases (
  id           NUMBER CONSTRAINT purchases_pk PRIMARY KEY,
  purchased_by REF t_customer SCOPE IS customers2,
  product      REF t_product  SCOPE IS products,
  quantity     NUMBER
);

-- insert sample data into customers table

INSERT INTO customers VALUES (
  t_customer(1, 'John', 'Smith', '01-JAN-1965', '650-555-1212',
    t_address('1 Anystreet', 'Anytown', 'CA', '12345')
  )
);

INSERT INTO customers VALUES (
  t_customer(2, 'Fred', 'Gilroy', '04-JUL-1958', '650-555-1212',
    t_address('2 Main Street', 'Middle Town', 'CA', '12345')
  )
);

-- insert sample data into customers2 table

INSERT INTO customers2 VALUES (
  t_customer(1, 'John', 'Smith', '01-JAN-1965', '650-555-1212',
    t_address('1 Anystreet', 'Anytown', 'CA', '12345')
  )
);

INSERT INTO customers2 VALUES (
  t_customer(2, 'Fred', 'Gilroy', '04-JUL-1958', '650-555-1212',
    t_address('2 Main Street', 'Middle Town', 'CA', '12345')
  )
);

-- insert sample data into customers3 table

INSERT INTO customers3 VALUES (
  1, 'John', 'Smith',
  t_address2(
    '1 Anystreet, Anytown, CA, 12345',
    '2 Main Street, Big Town, NY, 54321'
  )
);

INSERT INTO customers3 VALUES (
  2, 'Fred', 'Gilroy',
  t_address2(
    '3 Maple Street, Small Town, CA, 12345',
    '9 Winpole Avenue, Middle Town, NY, 54321'
  )
);

-- insert sample data into customers4 table

INSERT INTO customers4 VALUES (
  1, 'John', 'Smith',
  t_address3(
    t_address('1 Anystreet', 'Anytown', 'CA', '12345'),
    t_address('2 Main Street', 'Big Town', 'NY', '54321')
  )
);

INSERT INTO customers4 VALUES (
  2, 'Fred', 'Gilroy',
  t_address3(
    t_address('3 Maple Street', 'Small Town', 'CA', '12345'),
    t_address('9 Winpole Ave', 'Middle Town', 'NY', '54321')
  )
);

-- insert sample data into customers5 table

INSERT INTO customers5 VALUES (
  1, 'John', 'Smith',
  t_address4(
    '1 Anystreet, Anytown, CA, 12345',
    '2 Main Street, Big Town, NY, 54321'
  )
);

INSERT INTO customers5 VALUES (
  2, 'Fred', 'Gilroy',
  t_address4(
    '3 Maple Street, Small Town, CA, 12345',
    '9 Winpole Avenue, Middle Town, NY, 54321'
  )
);

-- insert sample data into products table

INSERT INTO products (id, name, description, price)
VALUES (1, 'Beyond Understanding', 'The frontiers of human knowledge', 19.95);

INSERT INTO products (id, name, description, price)
VALUES (2, 'Physics', 'Fundamental understanding of the natural world', 30.00);

INSERT INTO products (id, name, description, price)
VALUES (3, 'Star Travellers', 'The adventures of futuristic astronauts', 25.99);

INSERT INTO products (id, name, description, price)
VALUES (4, 'Seventh Sense', 'Supernatural tale of the unknown', 13.95);

-- insert sample data into purchases table

INSERT INTO purchases (
  id,
  purchased_by,
  product,
  quantity
) VALUES (
  1,
  (SELECT REF(c) FROM customers2 c WHERE c.id = 1),
  (SELECT REF(p) FROM products p WHERE p.id = 1),
  1
);

INSERT INTO purchases (
  id,
  purchased_by,
  product,
  quantity
) VALUES (
  2,
  (SELECT REF(c) FROM customers2 c WHERE c.id = 1),
  (SELECT REF(p) FROM products p WHERE p.id = 2),
  2
);

-- commit the transaction
COMMIT;